package dk.brics.xact;

/**
 * Visitor for nodes, including operation nodes.
 */
class AnyNodeVisitor {

	/**
	 * Constructs a new visitor.
	 */
	public AnyNodeVisitor() {}
	
	/**
	 * Visits an element node.
	 * Default implementation invokes {@link #visit(TempNode)}.
	 */
	public void visit(Element n) {
		visit((TempNode)n);
	}
	
	/**
	 * Visits a text node.
	 * Default implementation invokes {@link #visit(TempNode)}.
	 */
	public void visit(Text n) {
		visit((TempNode)n);
	}
	
	/**
	 * Visits a comment node.
	 * Default implementation invokes {@link #visit(TempNode)}.
	 */
	public void visit(Comment n) {
		visit((TempNode)n);
	}
	
	/**
	 * Visits a processing instruction node.
	 * Default implementation invokes {@link #visit(TempNode)}.
	 */
	public void visit(ProcessingInstruction n) {
		visit((TempNode)n);
	}
	
	/**
	 * Visits an attribute node.
	 * Default implementation invokes {@link #visit(AttrNode)}.
	 */
	public void visit(Attribute n) {
		visit((AttrNode)n);
	}
	
	/**
	 * Visits a template gap node.
	 * Default implementation invokes {@link #visit(TempNode)}.
	 */
	public void visit(TemplateGap n) {
		visit((TempNode)n);
	}
	
	/**
	 * Visits an attribute gap node.
	 * Default implementation invokes {@link #visit(AttrNode)}.
	 */
	public void visit(AttributeGap n) {
		visit((AttrNode)n);
	}
	
	/**
	 * Visits a namespace declaration node.
	 * Default implementation does nothing.
	 */
	public void visit(NamespaceDecl n) {}

	/**
	 * Visits a concat operation node.
	 * Default implementation invokes {@link #visit(OperationNode)}.
	 */
	public void visit(ConcatNode n) {
		visit((OperationNode)n);
	}
	
	/**
	 * Visits a plug-single operation node.
	 * Default implementation invokes {@link #visit(PlugNode)}.
	 */
	public void visit(PlugSingleNode n) {
		visit((PlugNode)n);
	}
	
	/**
	 * Visits a plug-list operation node.
	 * Default implementation invokes {@link #visit(PlugNode)}.
	 */
	public void visit(PlugListNode n) {
		visit((PlugNode)n);
	}
	
	/**
	 * Visits a plug operation node.
	 * Default implementation invokes {@link #visit(OperationNode)}.
	 */
	public void visit(PlugNode n) {
		visit((OperationNode)n);
	}
	
	/**
	 * Visits a close operation node.
	 * Default implementation invokes {@link #visit(OperationNode)}.
	 */
	public void visit(CloseNode n) {
		visit((OperationNode)n);
	}
	
	/**
	 * Visits an attribute-like node.
	 * Default implementation does nothing.
	 */
	public void visit(AttrNode n) {}
	
	/**
	 * Visits a template node.
	 * Default implementation invokes {@link #visit(XML)}.
	 */
	public void visit(TempNode n) {
		visit((XML)n);
	}
	
	/**
	 * Visits an operation node.
	 * Default implementation invokes {@link #visit(XML)}.
	 */
	public void visit(OperationNode n) {
		visit((XML)n);
	}
	
	/**
	 * Visits an XML node.
	 * Default implementation does nothing.
	 */
	public void visit(XML n) {}
}
