package dk.brics.xact;

import dk.brics.misc.Origin;

/**
 * Base class for attribute-like nodes.
 */
public abstract class AttrNode extends Node {

	private final String namespace;

	private final String localname;

	private final AttrNode nextattribute;

	/**
	 * Constructs a new attribute-like node.
	 */
	AttrNode(String namespace, String localname, AttrNode nextattribute, Origin origin) {
		super(origin);
		if (!StringTypes.isNCName(localname) || localname.equals("xmlns"))
			throw new XMLWellformednessException("illegal attribute name " + localname);
		this.namespace = namespace;
		this.localname = localname;
		this.nextattribute = nextattribute;
	}

	/**
	 * Returns the namespace URI.
	 */
	public String getNamespace() {
		return namespace;
	}

	/**
	 * Returns the local name.
	 */
	public String getLocalName() {
		return localname;
	}

	/**
	 * Returns the expanded name.
	 */
	public String getExpandedName() {
		return expanded(namespace, localname);
	}

	/**
	 * Returns the next attribute-like node.
	 */
	public AttrNode getNextAttr() {
		return follow(nextattribute);
	}
	
	/**
	 * Returns the next attribute node.
	 */
	public Attribute getNextAttribute() {
		AttrNode a = getNextAttr();
		while (a != null && !(a instanceof Attribute))
			a = a.getNextAttr();
		return (Attribute)a;
	}

	/**
	 * Returns a copy of this attribute with the given successor.
	 */
	abstract public AttrNode copy(AttrNode nextattribute);
}
