package dk.brics.xact;

import dk.brics.misc.Origin;

/**
 * Comment node.
 */
public class Comment extends TempNode {
	
	private final String value;

	/**
	 * Constructs a new comment node.
	 */
	public Comment(String value, XML nextsibling, Origin origin) {
		super(nextsibling, origin);
		if (!StringTypes.isChars(value) || value.contains("--"))
			throw new XMLWellformednessException("illegal comment");
		this.value = value;
	}

	/**
	 * Constructs a new comment node.
	 */
	public Comment(String value) {
		this(value, null, null);
	}

	/**
	 * Constructs a new comment node.
	 */
	public Comment(String value, Origin origin) {
		this(value, null, origin);
	}

	/**
	 * Returns the comment value.
	 */
	public String getValue() {
		return value;
	}

	@Override
	void visitNormalizedBy(NodeVisitor v) {
		v.visit(this);
	}
	
	@Override
	<R,A> R visitNormalizedBy(NodeVisitorParameterized<R,A> v, A arg) {
		return v.visit(this, arg);
	}
	
	@Override
	void visitAnyBy(AnyNodeVisitor v) {
		v.visit(this);
	}

	@Override
	public Comment copy(XML nextsibling) {
		return new Comment(value, nextsibling, getOrigin());
	}

	@Override
	public String toString() {
		return "[Comment]";
	}
}
