package dk.brics.xact;

import dk.brics.misc.Origin;

/**
 * Concatenation operation node.
 */
class ConcatNode extends OperationNode {

	private final XML left;
	
	private final XML right;

	/**
	 * Constructs a new concatenation operation node.
	 */
	public ConcatNode(XML left, XML right, Origin origin) {
		super(origin);
		this.left = left;
		this.right = right;
	}

	/**
	 * Returns the left node.
	 */
	public XML getLeftNode() {
		return follow(left);
	}

	/**
	 * Returns the right node.
	 */
	public XML getRightNode() {
		return follow(right);
	}
	
	/**
	 * Returns a copy of this node with the given left and right contents.
	 */
	public ConcatNode copy(XML left, XML right) {
		return new ConcatNode(left, right, getOrigin());
	}

	@Override
	void visitAnyBy(AnyNodeVisitor v) {
		v.visit(this);
	}

	@Override
	public String toString() {
		return "[ConcatNode]";
	}
}
