package dk.brics.xact;

import dk.brics.misc.Origin;
import dk.brics.xact.operations.XMLPrinter;

/**
 * Namespace declaration node.
 * Provides a namespace prefix for use by {@link XMLPrinter}.
 */
public class NamespaceDecl extends Node {

	private final String prefix;

	private final String namespace;
	
	private final NamespaceDecl nextnsdecl;

	/**
	 * Constructs a new namespace declaration node.
	 */
	public NamespaceDecl(String prefix, String namespace, NamespaceDecl nextnsdecl, Origin origin) {
		super(origin);
		if (prefix.length()>0 && (!StringTypes.isNCName(prefix) || prefix.equals("xmlns")))
			throw new XMLWellformednessException("illegal namespace prefix");
		if (prefix.equals("xml") && !namespace.equals("http://www.w3.org/XML/1998/namespace"))
			throw new XMLWellformednessException("illegal namespace declaration");
		this.prefix = prefix;
		this.namespace = namespace;
		this.nextnsdecl = nextnsdecl;
	}

	/**
	 * Constructs a new namespace declaration node.
	 */
	public NamespaceDecl(String prefix, String namespace) {
		this(prefix, namespace, null, null);
	}

	/**
	 * Constructs a new namespace declaration node.
	 */
	public NamespaceDecl(String prefix, String namespace, Origin origin) {
		this(prefix, namespace, null, origin);
	}

	/**
	 * Returns the namespace URI.
	 */
	public String getNamespace() {
		return namespace;
	}

	/**
	 * Returns the prefix.
	 */
	public String getPrefix() {
		return prefix;
	}
	
	/**
	 * Returns the next namespace declaration.
	 */
	public NamespaceDecl getNext() {
		return nextnsdecl;
	}

	@Override
	void visitNormalizedBy(NodeVisitor v) {
		v.visit(this);
	}
	
	@Override
	<R,A> R visitNormalizedBy(NodeVisitorParameterized<R,A> v, A arg) {
		return v.visit(this, arg);
	}
	
	@Override
	void visitAnyBy(AnyNodeVisitor v) {
		v.visit(this);
	}

	/**
	 * Returns a copy of this node with the given successor.
	 */
	NamespaceDecl copy(NamespaceDecl nextnsdecl) {
		return new NamespaceDecl(prefix, namespace, nextnsdecl, getOrigin());
	}
	
	@Override
	public String toString() {
		return "[NamespaceDecl " + prefix + " " + namespace + "]";
	}
}
