package dk.brics.xact;

import dk.brics.misc.Origin;

/**
 * Processing instruction node.
 */
public class ProcessingInstruction extends TempNode {
	
	private final String target;
	
	private final String data;

	/**
	 * Constructs a new processing instruction node.
	 */
	public ProcessingInstruction(String target, String data, XML nextsibling, Origin origin) {
		super(nextsibling, origin);
		if (!StringTypes.isName(target) || target.toLowerCase().equals("xml"))
			throw new XMLWellformednessException("illegal processing instruction target");
		if (!StringTypes.isChars(data) || data.contains("?>"))
			throw new XMLWellformednessException("illegal processing instruction data");
		this.target = target;
		this.data = data;
	}

	/**
	 * Constructs a new processing instruction node.
	 */
	public ProcessingInstruction(String target, String data) {
		this(target, data, null, null);
	}

	/**
	 * Constructs a new processing instruction node.
	 */
	public ProcessingInstruction(String target, String data, Origin origin) {
		this(target, data, null, origin);
	}

	/**
	 * Returns the target.
	 */
	public String getTarget() {
		return target;
	}

	/**
	 * Returns the data.
	 */
	public String getData() {
		return data;
	}

	@Override
	void visitNormalizedBy(NodeVisitor v) {
		v.visit(this);
	}
	
	@Override
	<R,A> R visitNormalizedBy(NodeVisitorParameterized<R,A> v, A arg) {
		return v.visit(this, arg);
	}
	
	@Override
	void visitAnyBy(AnyNodeVisitor v) {
		v.visit(this);
	}

	@Override
	public ProcessingInstruction copy(XML nextsibling) {
		return new ProcessingInstruction(target, data, nextsibling, getOrigin());
	}

	@Override
	public String toString() {
		return "[ProcessingInstruction " + target + "]";
	}
}
