package dk.brics.xact;

import dk.brics.misc.Origin;

/**
 * Template gap node.
 */
public class TemplateGap extends TempNode {

	private final String gap;
	
	private final String type;

	/**
	 * Constructs a new template gap node.
	 */
	public TemplateGap(String gap, String type, XML nextsibling, Origin origin) {
		super(nextsibling, origin);
		this.gap = gap;
		this.type = type;
	}

	/**
	 * Constructs a new template gap node.
	 */
	public TemplateGap(String gap, String type) {
		this(gap, type, null, null);
	}

	/**
	 * Constructs a new template gap node with no type.
	 */
	public TemplateGap(String gap) {
		this(gap, null, null, null);
	}

	/**
	 * Constructs a new template gap node.
	 */
	public TemplateGap(String gap, String type, Origin origin) {
		this(gap, type, null, origin);
	}

	/**
	 * Constructs a new template gap node with no type.
	 */
	public TemplateGap(String gap, Origin origin) {
		this(gap, null, null, origin);
	}

	/**
	 * Returns the gap name.
	 */
	public String getGap() {
		return gap;
	}
	
	/**
	 * Returns the gap type, or null if absent.
	 */
	public String getType() {
		return type;
	}

	@Override
	void visitNormalizedBy(NodeVisitor v) {
		v.visit(this);
	}
	
	@Override
	<R,A> R visitNormalizedBy(NodeVisitorParameterized<R,A> v, A arg) {
		return v.visit(this, arg);
	}
	
	@Override
	void visitAnyBy(AnyNodeVisitor v) {
		v.visit(this);
	}

	@Override
	public TemplateGap copy(XML nextsibling) {
		return new TemplateGap(gap, type, nextsibling, getOrigin());
	}
	
	@Override
	public String toString() {
		return "[TemplateGap " + (type != null ? type + " " : "") + gap + "]"; 
	}
}
