package dk.brics.xact;

import dk.brics.misc.Origin;

/**
 * Runtime exception caused by an invalid XML operation.
 */
@SuppressWarnings("serial")
public class XMLException extends RuntimeException {
	
	private final Origin origin;

	/**
	 * Constructs a new exception.
	 */
	public XMLException(String s) {
		this(s, null, null);
	}

	/**
	 * Constructs a new exception.
	 */
	public XMLException(String s, Throwable cause) {
		this(s, cause, null);
	}

	/**
	 * Constructs a new exception.
	 */
	public XMLException(Throwable cause) {
		this(cause.getMessage(), cause, null);
	}
	
	/**
	 * Constructs a new exception.
	 */
	public XMLException(String s, Origin origin) {
		this(s, null, origin);
	}

	/**
	 * Constructs a new exception.
	 */
	public XMLException(String s, Throwable cause, Origin origin) {
		super(s, cause);
		this.origin = origin;
	}

	/**
	 * Constructs a new exception.
	 */
	public XMLException(Throwable cause, Origin origin) {
		this(origin != null ? (cause.getMessage() + " at " + origin) : cause.getMessage(), cause, origin);
	}
	
	/**
	 * Returns a relevant source location, or null if not available.
	 */
	public Origin getOrigin() {
		return origin;
	}
}