package dk.brics.xact.analysis;

/**
 * Debug mode settings.
 * Enable debug info with <code>-Ddk.brics.xact.analysis.debug=&lt;LEVEL&gt;</code>.
 */
public class Debug { // TODO: generalize the Debug class from schematools
	
	private static int debug_level;
	
	private static int indent;
	
	static {
		String debug = System.getProperty("dk.brics.xact.analysis.debug");
		if (debug != null)
			try {
				debug_level = Integer.parseInt(debug);
			} catch (NumberFormatException e) {
				debug_level = 1;
			}
	}
	
	private Debug() {}
	
	/**
	 * Returns the debug level.
	 * @return debug level
	 */
	public static int getLevel() {
		return debug_level;
	}
	
	/**
	 * Sets the debug level.
	 */
	public static void setLevel(int level) {
		debug_level = level;
	}
	
	/**
	 * Resets indentation level.
	 */
	public static synchronized void resetIndent() {
		indent = 0;
	}
	
	private static void ind() {
		StringBuilder s = new StringBuilder();
		for (int i = 0; i < indent; i++)
			s.append(' ');
		System.err.print(s);
	}
	
	/**
	 * Prints to <code>System.err</code> if debug is enabled.
	 * @param level minimal debug level required to print the string
	 * @param indent indent if true
	 * @param s string
	 */
	public static synchronized void println(int level, boolean indent, String s) {
		if (debug_level >= level) {
			if (indent)
				ind();
			System.err.println(s);
		}
	}

	/**
	 * Prints to <code>System.err</code> if debug is enabled.
	 * @param level minimal debug level required to print the string
	 * @param indent indent if true
	 * @param s string
	 */
	public static synchronized void print(int level, boolean indent, String s) {
		if (debug_level >= level) {
			if (indent)
				ind();
			System.err.print(s);
			System.err.flush();
		}
	}
	
	/**
	 * Increment indentation level.
	 */
	public static synchronized void inc() {
		indent++;
	}
	
	/**
	 * Decrement indentation level.
	 */
	public static synchronized void dec() {
		indent--;
	}
}
