package dk.brics.xact.analysis;

import java.util.List;

import dk.brics.xact.analysis.flowgraph.FlowGraph;
import dk.brics.xact.analysis.transformations.ArrayTransformer;
import dk.brics.xact.analysis.transformations.DefUseTransformer;

/**
 * Object that gets notified of certain events in the XML analysis,
 * mostly for the purpose of debugging and benchmarking the analysis.
 */
public interface Diagnostics {
	/** Called just after the Soot classes have been loaded */
	void afterSootLoaded();
	
	/** Called just after the control-flow graph has been built and no transformer has run yet */
	void afterControlFlow(FlowGraph g);
	
	/** Called just after call statements have been removed and replaced by interprocedural edges */
	void afterCalls(FlowGraph g);
	
	/** Called just after unreachable nodes have been removed */
	void afterUnreachable(FlowGraph g);
	
	/** Called just after the {@link ArrayTransformer} completes */
	void afterArrays(FlowGraph g);
	
	/** Called just after the {@link DefUseTransformer} completes */
	void afterDefUse(FlowGraph g);
	
	/** Called just after the flow graph has been split */
	void afterSplit(FlowGraph g, List<FlowGraph> subgraphs);
	
	/** Called after XML Schemas have been loaded and linked */
    void afterSchemaTypeLinking(FlowGraph g);
	
	/**
	 * Implementation of this interface where every method does nothing.
	 */
	public static final Diagnostics NULL = new Diagnostics() {
		public void afterSootLoaded() {
		}
		public void afterControlFlow(FlowGraph g) {
		}
		public void afterArrays(FlowGraph g) {
		}
		public void afterDefUse(FlowGraph g) {
		}
		public void afterCalls(FlowGraph g) {
		}
		public void afterUnreachable(FlowGraph g) {
		}
		public void afterSplit(FlowGraph g, List<FlowGraph> subgraphs) {
		}
		public void afterSchemaTypeLinking(FlowGraph g) {
		}
	};


}
