package dk.brics.xact.analysis.concurrent;

import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

/**
 * Adapter for {@link ExecutorService} to {@link XactTaskRunner}.
 */
public class ExecutorTaskRunner implements XactTaskRunner {
    private boolean running=false;
    private ExecutorService service;
    private LinkedList<Callable<Object>> tasks = new LinkedList<Callable<Object>>();
    public ExecutorTaskRunner(ExecutorService service) {
        this.service = service;
    }
    public void addTask(Runnable runnable) {
        if (running)
            throw new IllegalStateException("Cannot add tasks while running");
        tasks.add(Executors.callable(runnable));
    }
    public void runTasks() {
        if (running) {
            throw new IllegalStateException("runTasks is already running");
        }
        running=true;
        try {
            List<Future<Object>> list = service.invokeAll(tasks);
            tasks = new LinkedList<Callable<Object>>();
            for (Future<Object> f : list) {
                f.get();
            }
        } catch (InterruptedException ex) {
            Thread.currentThread().interrupt();
            throw new RuntimeException(ex);
        } catch (ExecutionException ex) {
            throw new RuntimeException(ex.getCause());
        } finally {
            running=false;
        }
    }
    public boolean maybeMultiThreaded() {
        return true;
    }
}
