package dk.brics.xact.analysis.concurrent;

/**
 * Used by the analysis to schedule independant tasks.
 * <p/>
 * A task runner can be in one of two states:
 * <ol>
 * <li><i>Accepting tasks</i>: Tasks can be added, but are not being executed.
 * <li><i>Executing tasks</i>: Tasks are being executed, and new tasks cannot be added.
 * </ol>
 */
public interface XactTaskRunner {
    /**
     * Adds a tasks to the task scheduler. The task will not be run until
     * the next invocation of {@link #runTasks()}.
     * @param runnable the task to be executed
     */
    void addTask(Runnable runnable);
    
    /**
     * Runs all scheduled tasks and returns when all have completed. The queue of
     * tasks is cleared after this.
     * <p/>
     * Throws a runtime exception if any of the tasks terminated with an unhandled
     * exception.
     */
    void runTasks();
    
    /**
     * Returns true if the scheduler might be multithreaded, or false if it
     * is definitely not multithreaded.
     * <p/>
     * If not multithreaded, the analysis can occasionally skip certain tasks 
     * that only exist to avoid interference.
     * @return false if definitely single-threaded, otherwise true
     */
    boolean maybeMultiThreaded();
}
