package dk.brics.xact.analysis.config;

import java.net.URL;
import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutorService;

import soot.SootClass;
import soot.SootField;
import soot.SootMethod;
import soot.ValueBox;
import soot.jimple.InvokeExpr;
import dk.brics.automaton.Automaton;

public class StandardConfiguration implements Configuration {
	
	private String basedir = null;
	
	public String getBasedir() {
		return basedir;
	}

	public void setBasedir(String basedir) {
		basedir = basedir.replace("\\", "/");
		if (!basedir.endsWith("/"))
			basedir = basedir + "/";
		this.basedir = basedir;
	}

	public boolean canCallExtern(InvokeExpr expr) {
		return !expr.getMethod().getDeclaringClass().isApplicationClass();
	}

	public boolean isExternallyCallable(SootMethod method) {
		String methodName = method.getName();
		String subsig = method.getSubSignature();
		// initializers
		if (methodName.equals("<clinit>") || methodName.equals("<init>")) {
			return true;
		}
		// main method
		if (method.isPublic() && method.isStatic() && subsig.equals("void main(java.lang.String[])")) {
			return true;
		}
		// equals() and hashCode() and toString() are often called by library functions
		if (subsig.equals("boolean equals(java.lang.Object)") || subsig.equals("int hashCode()")
			|| subsig.equals("java.lang.String toString()")) {
			return true;
		}
		
		return false;
	}
	
	public boolean hasExternalSubtypes(SootClass clazz) {
		return true;
	}
	
	public boolean isNormalCollection(String qualifiedName) {
		return qualifiedName.startsWith("java.util");
	}
	
	public String getMethodParameterType(SootMethod method, int param, String annotation) {
		return annotation;
	}
	
	public String getMethodReturnType(SootMethod method, String annotation) {
		return annotation;
	}
	
	public String getFieldType(SootField field, String annotation) {
		return annotation;
	}
	
	public void modifyNamespaces(Map<String, String> namespaces) {
	}
	
	public Collection<URL> getAdditionalSchemas() {
		return Collections.emptyList();
	}
	
	public Automaton resolveToString(SootClass clazz) {
		return null;
	}
	
	public String resolveToXML(SootMethod toXmlMethod) {
		return null;
	}
	
	public ExecutorService getExecutorService() {
	    return null;
	}
	
	public String translateSchemaLocation(String location) {
		if (basedir != null && location.startsWith("file:")) {
			return "file:" + basedir + location.substring("file:".length());
		} else {
			return location;
		}
	}
	
	public Set<ValueBox> getHotspots() {
		return Collections.<ValueBox>emptySet();
	}
}
