package dk.brics.xact.analysis.dataflow;

import java.util.Collection;

import dk.brics.xact.analysis.flowgraph.FlowGraph;
import dk.brics.xact.analysis.flowgraph.Statement;
import dk.brics.xact.analysis.flowgraph.VariableFilter;

/**
 * Lattice and transfer functions for a dataflow analysis. 
 */
public interface AnalysisInterface<ElementType> {
	
	/**
	 * Constructs a new lattice bottom element.
	 */
    public ElementType newBottomElement();

    /** 
     * Computes the least upper bound of two lattice elements.
     * @param source the first element
     * @param filter the filter to merge through
     * @param dest the second element and destination for the result 
     * @return <code>true</code> if the new value of <code>dest</code> is changed
     */
    public boolean merge(ElementType source, VariableFilter filter, ElementType dest);

    /** 
     * Computes the lattice element resulting from flowing through the given statement.
     * @param in the flow before the statement
     * @param out destination for the result
     * @return <code>true</code> if the new value of <code>out</code> is changed
     */
    public boolean transfer(ElementType in, Statement s, ElementType out);
    
    /**
     * Returns true if this is a forwards analysis, and false if it is a backwards analysis.
     */
    public boolean isForward();
    
    /**
     * Returns the set of statements to start the analysis at. For forwards analyses, the
     * graph's entry points are usually the correct return value.
     * @param g the flow graph about to be analyzed 
     * @return an unmodifiable set of statements.
     */
    public Collection<Statement> getInitial(FlowGraph g);
}
