package dk.brics.xact.analysis.flowgraph;

/**
 * Flow graph edge. 
 */
public class Edge<Nodetype,Edgetype> {

	private Nodetype from;
	
	private Nodetype to;
	
	private Edgetype data;
	
	/** 
	 * Constructs an edge.
	 */
	public Edge(Nodetype from, Nodetype to, Edgetype data) {
		this.from = from;
		this.to = to;
		this.data = data;
	}
	
	/** 
	 * Returns the source node.
	 */
	public Nodetype getFrom() {
		return from;
	}
	
	/** 
	 * Returns the target node.
	 */
	public Nodetype getTo() {
		return to;
	}
	
	/** 
	 * Returns the edge data.
	 */
	public Edgetype getData() {
		return data;
	}
	
	/**
	 * Checks whether the given edge is equal to this edge.
	 */
	@Override
	public boolean equals(Object other) {
		if (other instanceof Edge<?,?>) {
			Edge<?,?> o = (Edge<?,?>)other;
			return from.equals(o.from) && to.equals(o.to) && data.equals(o.data);
		} else 
			return false;
	}
	
	/**
	 * Returns a hash code for this edge.
	 */
	@Override
	public int hashCode() {
		return from.hashCode() + to.hashCode() + data.hashCode();
	}
}
