package dk.brics.xact.analysis.flowgraph;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import dk.brics.misc.Origin;
import dk.brics.xmlgraph.Node;

/**
 * Schema type.
 */
public class SchemaType extends Entity {

	private String type; // expanded name
	
	private Collection<String> tgaps;
	
	private Collection<String> agaps;
	
	private Map<String,String> gap_types;
	private Map<String,Node> gapTypeNodes = new HashMap<String, Node>();
	
	private Node typeNode;
	
	/**
	 * Constructs a new schema type.
	 */
	public SchemaType(String type, Collection<String> tgaps, Collection<String> agaps, 
			Map<String,String> gap_types, Origin origin) {
		super(origin);
		assert gap_types.keySet().containsAll(tgaps);
		assert gap_types.keySet().containsAll(agaps);
		this.type = type;
		this.tgaps = tgaps;
		this.agaps = agaps;
		this.gap_types = gap_types;
	}
	
	/**
	 * Constructs a new schema type with no gap info.
	 */
	public SchemaType(String type, Origin origin) {
		this(type, 
	        Collections.<String>emptySet(), 
	        Collections.<String>emptySet(), 
	        Collections.<String,String>emptyMap(), origin);
	}
	
	/**
	 * Returns whether this schema type has gap annotations, equivalent
	 * to <code>!getGapTypes().isEmpty()</code>.
	 */
	public boolean hasGapAnnotations() {
	    return !gap_types.isEmpty();
	}
	
	/**
	 * Returns the type string.
	 */
	public String getType() {
		return type;
	}
	
	/**
	 * Returns the collection of template gaps,
	 */
	public Collection<String> getTemplateGaps() {
		return tgaps;
	}
	
	/**
	 * Returns the collection of attribute gaps, 
	 */
	public Collection<String> getAttributeGaps() {
		return agaps;
	}
	
	/**
	 * Returns the gap type map,
	 */
	public Map<String,String> getGapTypes() {
		return gap_types;
	}
	
	public Map<String, Node> getGapTypeNodes() {
        return gapTypeNodes;
    }
	
	public Node getTypeNode() {
        return typeNode;
    }
	public void setTypeNode(Node typeNode) {
        this.typeNode = typeNode;
    }
	
}
