package dk.brics.xact.analysis.flowgraph;

import dk.brics.misc.Origin;
import dk.brics.xact.analysis.flowgraph.statements.StatementVisitor;

/** 
 * Superclass of abstract statements.
 */
public abstract class Statement extends Entity {
	
	/** 
	 * Constructs a new <code>Statement</code> object.
	 */
	public Statement(Origin origin) {
		super(origin);
	}
	
	/** 
	 * Visits this statement by the given visitor.
	 */
	public abstract void visitBy(StatementVisitor v);
	
	/** 
	 * Returns a textual description of the statement.
	 * This includes the origin and other statically available information.
	 */
	@Override
	public String toString() {
		return getOrigin().getFile() + " line " + getOrigin().getLine();
	}
	
	/**
	 * Returns the name of the operation.
	 */
	abstract public String getOpName();
}
