package dk.brics.xact.analysis.flowgraph.statements;

import dk.brics.misc.Origin;
import dk.brics.xact.XML;
import dk.brics.xact.analysis.config.Configuration;
import dk.brics.xact.analysis.flowgraph.SchemaType;
import dk.brics.xact.analysis.flowgraph.Variable;

/**
 * Assignment of {@link dk.brics.xact.XML#analyze(String)} or assignment to a type-annotated variable, parameter or return value.
 */
public class AnalyzeStm extends InstanceAssignment {
	
	private SchemaType schema;

	private String type;// expanded qname
	
	private Kind kind;
	
	public enum Kind {
		/**
		 * Call to {@link XML#analyze(String)}.
		 */
		ANALYZECALL,
		
		/**
		 * Assignment to some type annotation.
		 */
		TYPEANNOTATION,
		
		/**
		 * A hotspot added manually using {@link Configuration#getHotspots()}.
		 * These should not be checked by the XML analysis, but the XML graph must be available.
		 */
		HOTSPOT,
	}
	
	/**
	 * Constructs a new <code>AnalyzeStm</code> object.
	 */
	public AnalyzeStm(Kind kind, Variable dest, Variable base, SchemaType schema, Origin origin) {
		super(dest, base, origin);
		this.kind = kind;
		this.schema = schema;
		this.type = schema != null ? schema.getType() : "-";
	}
	
	public Kind getKind() {
		return kind;
	}
	public void setKind(Kind kind) {
		this.kind = kind;
	}

	/**
	 * Returns the schema.
	 */
	public SchemaType getSchema() {
		return schema;
	}
	
	@Override
	public void visitBy(StatementVisitor v) { 
		v.visitAnalyzeStm(this); 
	}
	
	@Override
	public String toString() {
		return "analyze(" + getBase() + "," + type + ")" + super.toString();
	}	
	
	@Override
	public String getOpName() {
		return "analyze";
	}
}
