package dk.brics.xact.analysis.flowgraph.statements;

import dk.brics.misc.Origin;
import dk.brics.xact.analysis.flowgraph.Statement;
import dk.brics.xact.analysis.flowgraph.Variable;

/** 
 * Check operation for 
 * {@link dk.brics.xact.Element#getNumber(String)},
 * {@link dk.brics.xact.Element#getString(String)},
 * {@link dk.brics.xact.Element#has(String)},
 * {@link dk.brics.xact.Element#toDocument()},
 * {@link dk.brics.xact.Element#isElement()},
 * and variants.
 */
public class CheckStm extends Statement {
	
	/**
	 * Node kinds.
	 */
	public enum Kind {GETNUMBER, GETSTRING, HAS, TODOCUMENT, ISATTRIBUTE, ISELEMENT, ISTEXT};
	
	private Kind kind;

	/** 
	 * The XML variable containing the base value.
	 */
	private Variable base;

	/** 
	 * The XPath expression, or null if none.
	 */
	private String xpath;
	
	/** 
	 * Constructs a new <code>CheckStm</code> object.
	 * <code>xpath</code> and <code>base</code> can be null.
	 */
	public CheckStm(Kind kind, Variable base, String xpath, Origin origin) {
		super(origin);
		this.kind = kind;
		this.base = base;
		this.xpath = xpath;
	}
	
	/**
	 * Returns the kind.
	 */
	public Kind getKind() {
		return kind;
	}
	
	/**
	 * Returns the XPath expression, or null if none is available.
	 */
	public String getXPath() {
		return xpath;
	}
	
	/**
	 * Returns the base variable, or null if none is available.
	 */
	public Variable getBase() {
		return base;
	}
	
	/**
	 * Sets the base variable.
	 */
	public void setBase(Variable base) {
		this.base = base;
	}
	
	@Override
	public void visitBy(StatementVisitor v) { 
		v.visitCheckStm(this); 
	}
	
	@Override
	public String toString() {
		return "check[" + kind + "](" 
		+ (base!=null ? base : "")
		+ (xpath!=null ? "," + xpath : "")
		+ ")" + " at " + super.toString();
	}
	
	@Override
	public String getOpName() {
		switch (kind) {
		case GETNUMBER:
			return "getNumber";
		case GETSTRING:
			return "getString";
		case HAS:
			return "has";
		case ISATTRIBUTE:
			return "isAttribute";
		case ISELEMENT:
			return "isElement";
		case ISTEXT:
			return "isText";
		case TODOCUMENT:
			return "toDocument";
		default:
			throw new RuntimeException("Unexpected statement kind " + kind);
		}
	}
}