package dk.brics.xact.analysis.flowgraph.statements;

import dk.brics.misc.Origin;
import dk.brics.xact.analysis.flowgraph.SchemaType;
import dk.brics.xact.analysis.flowgraph.Variable;

/**
 * Assignment of {@link dk.brics.xact.XML#gapify(String, String)} and variants.
 */
public class GapifyStm extends InstanceAssignment {

	/** 
	 * The XPath expression.
	 */
	private String xpath;
	
	/** 
	 * The gap name.
	 */
	private String gapname;
	
	private SchemaType schema;
	
	/** 
	 * Constructs a new <code>GapifyStm</code> object.
	 * <code>type</code> and <code>schema</code> can be null.
	 */
	public GapifyStm(Variable dest, Variable base, String xpath, String gapname, SchemaType schema, Origin origin) {
		super(dest, base, origin);
		this.xpath = xpath;
		this.gapname = gapname;
		this.schema = schema;
	}

	/**
	 * Returns the XPath expression, or null if none is available.
	 */
	public String getXPath() {
		return xpath;
	}
	
	/**
	 * Returns the gap name.
	 */
	public String getGapName() {
		return gapname;
	}
	
	/**
	 * Returns the schema.
	 */
	public SchemaType getSchema() {
		return schema;
	}
	
	@Override
	public void visitBy(StatementVisitor v) { 
		v.visitGapifyStm(this); 
	}
	
	@Override
	public String toString() {
		return "gapify(" + getBase() + "," + xpath + "," 
		+ gapname + (schema != null ? "," + schema.getType() : "") + ")" + super.toString();
	}
	
	@Override
	public String getOpName() {
		return "gapify";
	}
}
