package dk.brics.xact.analysis.flowgraph.statements;

import dk.brics.automaton.Automaton;
import dk.brics.misc.Origin;
import dk.brics.xact.analysis.flowgraph.Variable;

/**
 * Assignment of {@link dk.brics.xact.XML#insertAfter(String, Object)},
 * {@link dk.brics.xact.XML#appendContent(String, Object)},
 * {@link dk.brics.xact.XML#prependContent(String, Object)},
 * and variants.
 */
public class InsertStm extends InstanceAssignment {
	
	/**
	 * Node kinds.
	 */
	public enum Kind {APPEND, APPENDCONTENT, INSERTAFTER, INSERTBEFORE, PREPEND, PREPENDCONTENT, SETATTRIBUTE};
	
	private Kind kind;

	/** 
	 * The XPath expression, or null if none.
	 */
	private String xpath;
	
	/** 
	 * The XML variable containing the XML values being inserted.
	 */
	private Variable xmlsrc;
	
	/**
	 * The automaton containing the string values being plugged in.
	 */
	private Automaton stringsrc;
	
	/** 
	 * Constructs a new <code>InsertStm</code> object.
	 * <code>xpath</code> can be null.
	 */
	public InsertStm(Kind kind, Variable dest, Variable base, String xpath, Automaton stringsrc, Variable xmlsrc, Origin origin) {
		super(dest, base, origin);
		this.kind = kind;
		this.xpath = xpath;
		this.stringsrc = stringsrc;
		this.xmlsrc = xmlsrc;
	}

	/**
	 * Returns the kind.
	 */
	public Kind getKind() {
		return kind;
	}
	
	/**
	 * Returns the string source automaton.
	 */
	public Automaton getStringSource() {
		return stringsrc;
	}

	public void setStringSource(Automaton stringsrc) {
		this.stringsrc = stringsrc;
	}

	/**
	 * Returns the XML source variable.
	 */
	public Variable getXMLSource() {
		return xmlsrc;
	}
	
	/**
	 * Sets the XML source variable.
	 */
	public void setXMLSource(Variable xmlsrc) {
		this.xmlsrc = xmlsrc;
	}
	
	/**
	 * Returns the XPath expression.
	 */
	public String getXPath() {
		return xpath;
	}
	
	@Override
	public void visitBy(StatementVisitor v) { 
		v.visitInsertStm(this); 
	}
	
	@Override
	public String toString() {
		return "insert[" + kind + "](" + getBase() + (xpath!=null ? "," + xpath : "") 
			+ "," + xmlsrc + ";" + stringsrc.getNumberOfStates() + "states)" + super.toString();
	}
	
	@Override
	public String getOpName() {
		switch (kind) {
		case APPEND:
			return "append";
		case APPENDCONTENT:
			return "appendContent";
		case INSERTAFTER:
			return "insertAfter";
		case INSERTBEFORE:
			return "insertBefore";
		case PREPEND:
			return "prepend";
		case PREPENDCONTENT:
			return "prependContent";
		case SETATTRIBUTE:
			return "setAttribute";
		default:
			throw new RuntimeException("Unexpected statement kind " + kind);
		}
	}
}
