package dk.brics.xact.analysis.flowgraph.statements;

import dk.brics.automaton.Automaton;
import dk.brics.misc.Origin;
import dk.brics.xact.analysis.flowgraph.Variable;

/** 
 * Assignment of a new {@link dk.brics.xact.Element#Element(String, String, AttrNode, XML, NamespaceDecl, XML, Origin)} and variants.
 */
public class NodeStm extends Assignment {
	
	/**
	 * Node kinds.
	 */
	public enum Kind {ELEMENT, ATTRIBUTE, ATTRIBUTEGAP, COMMENT, PROCESSINGINSTRUCTION, TEMPLATEGAP, TEXT};
	
	private Kind kind;

	/**
	 * First child, or null if none.
	 */
	private Variable firstchild;
	
	/**
	 * First attribute or attribute gap, or null if none.
	 */
	private Variable firstattr;
	
	/**
	 * Next sibling or attribute, or null if none.
	 */
	private Variable nextnode;
	
	private Automaton name;
	
	private Automaton value;
	
	private String gap;
	
	private String type;

	/** 
	 * Constructs a new <code>NodeStm</code> object.
	 * <code>firstchild</code>, <code>firstattr</code>, <code>gap</code>,
	 * <code>type</code> and <code>nextnode</code> can be null.
	 */
	public NodeStm(Kind kind, Variable dest, Automaton name, Automaton value, 
			Variable firstchild, Variable firstattr, String gap, String type, 
			Variable nextnode, Origin origin) {
		super(dest, origin);
		this.kind = kind;
		this.name = name;
		this.value = value;
		this.firstchild = firstchild;
		this.firstattr = firstattr;
		this.gap = gap;
		this.type = type;
		this.nextnode = nextnode;
	}
	
	/**
	 * Returns the kind.
	 */
	public Kind getKind() {
		return kind;
	}
	
	/**
	 * Returns the element/attribute name, or null if none is available.
	 */
	public Automaton getName() {
		return name;
	}
	
	/**
	 * Returns the attribute value, or null if none is available.
	 */
	public Automaton getValue() {
		return value;
	}
	
	
	
	public void setName(Automaton name) {
		this.name = name;
	}

	public void setValue(Automaton value) {
		this.value = value;
	}

	/**
	 * Returns the first-child variable, or null if none is available.
	 */
	public Variable getFirstChild() {
		return firstchild;
	}
	
	/**
	 * Sets the first-child variable.
	 */
	public void setFirstChild(Variable firstchild) {
		this.firstchild = firstchild;
	}
	
	/**
	 * Returns the first-attr variable, or null if none is available.
	 */
	public Variable getFirstAttr() {
		return firstattr;
	}
	
	/**
	 * Sets the first-attr variable.
	 */
	public void setFirstAttr(Variable firstattr) {
		this.firstattr = firstattr;
	}
	
	/**
	 * Returns the next-node variable, or null if none is available.
	 */
	public Variable getNextNode() {
		return nextnode;
	}
	
	/**
	 * Sets the next-node variable.
	 */
	public void setNextNode(Variable nextnode) {
		this.nextnode = nextnode;
	}
	
	/**
	 * Returns the gap name, or null if none is available.
	 */
	public String getGap() {
		return gap;
	}
	
	/**
	 * Returns the gap type, or null if none is available.
	 */
	public String getType() {
		return type;
	}

	@Override
	public void visitBy(StatementVisitor v) { 
		v.visitNodeStm(this); 
	}
	
	@Override
	public String toString() {
		return "node[" + kind + "]" + super.toString();
	}
	
	@Override
	public String getOpName() {
		switch (kind) {
		case ATTRIBUTE:
			return "Attribute";
		case ATTRIBUTEGAP:
			return "AttributeGap";
		case ELEMENT:
			return "Element";
		case COMMENT:
			return "Comment";
		case PROCESSINGINSTRUCTION:
			return "ProcessingInstruction";
		case TEMPLATEGAP:
			return "TemplateGap";
		case TEXT:
			return "Text";
		default:
			throw new RuntimeException("Unexpected statement kind " + kind);
		}
	}
}
