package dk.brics.xact.analysis.xmlgraph;

/**
 * A generic lattice denoting whether something is empty or non-empty.
 */
public enum EPresence {
	UNKNOWN,
	EMPTY,
	NONEMPTY,
	BOTTOM;
	
	// XXX EPresence is isomorphic to EBooleanLattice. Replace this class entirely? 
	// CONCAT is the same as either AND or OR depending on whether YES means EMPTY or NONEMPTY
	
	/**
	 * Returns whether the result is empty when concatenating the two elements.
	 * For example, "EMPTY concat EMPTY" is EMPTY, and "EMPTY concat NONEMPTY" is NONEMPTY.
	 */
	public EPresence concat(EPresence b) {
		EPresence a = this;
		if (a == BOTTOM || b == BOTTOM)
			return BOTTOM;
		if (a == EPresence.EMPTY)
			return b;
		if (b == EPresence.EMPTY)
			return a;
		if (a == NONEMPTY || b == NONEMPTY)
			return NONEMPTY;
		return EPresence.UNKNOWN;
	}
	public EPresence leastUpperBound(EPresence b) {
		EPresence a = this;
		if (a == BOTTOM)
			return b;
		if (b == BOTTOM)
			return a;
		if (a != b)
			return EPresence.UNKNOWN;
		return a;
	}
	public boolean definitelyEmpty() {
		return this == EMPTY || this == BOTTOM;
	}
	public boolean definitelyNonEmpty() {
		return this == NONEMPTY || this == BOTTOM;
	}
	public boolean maybeNonEmpty() {
		return this == NONEMPTY || this == UNKNOWN;
	}
	public boolean maybeEmpty() {
		return this == EMPTY || this == UNKNOWN;
	}
}
