package dk.brics.xact.analysis.xmlgraph;

import java.util.Set;

import dk.brics.xmlgraph.AttributeNode;
import dk.brics.xmlgraph.ElementNode;
import dk.brics.xmlgraph.MultiContentNode;
import dk.brics.xmlgraph.Node;
import dk.brics.xmlgraph.NodeProcessor;
import dk.brics.xmlgraph.OneOrMoreNode;
import dk.brics.xmlgraph.XMLGraph;

/**
 * Determines for each reachable node whether it always occurs as root when unfolded (yes),
 * or never occurs as root (no).
 */
public class RootAnalysis extends ForwardsXGAnalyzer<EBooleanLattice> {
    
    public RootAnalysis(XMLGraph xg) {
        super(xg);
        doAnalysis();
    }
    
    @Override
    protected EBooleanLattice bottom() {
        return EBooleanLattice.BOTTOM;
    }

    @Override
    protected EBooleanLattice initial(Node node) {
        return EBooleanLattice.YES;
    }
    
    @Override
    protected Set<Integer> initialNodes() {
        return getXmlGraph().getRoots();
    }
    
    @Override
    protected void transfer(EBooleanLattice src, Node node) {
        processor.src = src;
        node.process(processor);
    }
    
    private Processor processor = new Processor();
    private final class Processor extends NodeProcessor<Object> {
        EBooleanLattice src;
        
        @Override
        public Object process(ElementNode n) {
            merge(n.getContent(), EBooleanLattice.NO);
            return this;
        }
        @Override
        public Object process(AttributeNode n) {
            merge(n.getContent(), EBooleanLattice.NO);
            return this;
        }
        @Override
        public Object process(MultiContentNode n) {
            for (int child : n.getContents()) {
                merge(child, src);
            }
            return this;
        }
        @Override
        public Object process(OneOrMoreNode n) {
            merge(n.getContent(), src);
            return this;
        }
        
        private void merge(int target, EBooleanLattice lat) {
            put(target, get(target).leastUpperBound(lat));
        }
    }
    
}
