package dk.brics.xact.compiler;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.List;

/**
 * File utilitity methods.
 */
public class FileUtil {
	
	/**
	 * Finds all files with the given extension.
	 * @param dir directory to search
	 * @param subdirs if true, also search in subdirectories
	 * @param extension extension to include
	 */
	public static List<File> findAllFilesWithExtensions(File dir, boolean subdirs, String... extension) {
		if (!dir.isDirectory())
			throw new IllegalArgumentException(dir + " is not a directory");
		List<File> list = new ArrayList<File>();
		collectFilesWithExtensions(dir, subdirs, extension, list);
		return list;
	}
	
	private static void collectFilesWithExtensions(File dir, final boolean subdirs, final String[] extensions, List<File> list) {
		File[] files = dir.listFiles(new FileFilter() {
			public boolean accept(File pathname) {
                boolean match = false;
                for (String extension : extensions) {
				    match |= (pathname.getName().endsWith(extension) || (subdirs && pathname.isDirectory()));
                }
                return match;
			}
		});
		for (File file : files) {
			if (file.isDirectory()) {
				collectFilesWithExtensions(file, true, extensions, list);
			} else {
				list.add(file);
			}
		}
	}
}
