package dk.brics.xact.wrappers;

import dk.brics.xact.AttrNode;
import dk.brics.xact.AttributeGap;

/**
 * Wrapper for {@link AttrNode}.
 */
public abstract class AttrNodeWrapper<T extends AttrNode> extends NodeWrapper<T> {
	
	private AttrNodeWrapper<? extends AttrNode> nextattribute;

	private boolean touched_nextattribute;
	
	/**
	 * Constructs a new wrapper.
	 */
	public AttrNodeWrapper(T real, ElementWrapper parent) {
		super(real, parent);
	}
	
	/**
	 * Returns the next wrapped attribute-like node.
	 */
	public AttrNode getRealNextAttr() {
		if (nextattribute != null)
			return nextattribute.getReal();
		else
			return getReal().getNextAttr();
	}
	
	/**
	 * Returns the next attribute-like wrapper node.
	 */
	public AttrNodeWrapper<? extends AttrNode> getNextAttrWrapper() {
		return nextattribute;
	}
	
	/**
	 * Returns the next attribute wrapper node.
	 */
	public AttributeWrapper getNextAttribute() {
		if (!touched_nextattribute) {
			nextattribute = AttrNodeWrapper.make(getReal().getNextAttr(), getParent());
			touched_nextattribute = true;
		}
		AttrNodeWrapper<? extends AttrNode> a = nextattribute;
		if (a != null && !(a instanceof AttributeWrapper))
			a = a.getNextAttribute();
		return (AttributeWrapper)a;
	}

	/**
	 * Makes a wrapper for the given attribute-like node.
	 */
	public static AttrNodeWrapper<? extends AttrNode> make(AttrNode a, ElementWrapper parent) {
		AttrNodeWrapper<? extends AttrNode> res;
		if (a == null)
			res = null;
		else if (a.isAttribute())
			res = new AttributeWrapper(a.asAttribute(), parent);
		else
			res = new AttributeGapWrapper((AttributeGap)a, parent);
		return res;
	}
}
