package dk.brics.xact.wrappers;

import dk.brics.xact.AttrNode;
import dk.brics.xact.Element;
import dk.brics.xact.TempNode;
import dk.brics.xact.XML;

/**
 * Wrapper for {@link Element}.
 */
public class ElementWrapper extends ConcreteTempNodeWrapper<Element> {
	
	private TempNodeWrapper<? extends TempNode> firstchild;
	
	private AttrNodeWrapper<? extends AttrNode> firstattribute;
	
	private boolean touched_firstchild;
	
	private boolean touched_firstattribute;
	
	/**
	 * Constructs a new wrapper.
	 */
	public ElementWrapper(Element real, ElementWrapper parent) {
		super(real, parent);
	}
	
	/**
	 * Returns the first attribute-like node.
	 */
	public AttrNode getRealFirstAttr() {
		if (firstattribute != null)
			return firstattribute.getReal();
		else
			return getReal().getFirstAttr();
	}

	/**
	 * Returns the first child node.
	 */
	public XML getRealFirstChild() {
		if (firstchild != null)
			return firstchild.getReal();
		else
			return getReal().getFirstChild();
	}
	
	/**
	 * Returns the first child node wrapper.
	 */
	public TempNodeWrapper<? extends TempNode> getFirstChildWrapper() {
		return firstchild;
	}
	
	/**
	 * Returns the first attribute-like node wrapper.
	 */
	public AttrNodeWrapper<? extends AttrNode> getFirstAttrWrapper() {
		return firstattribute;
	}
	
	/**
	 * Returns the first concrete child node wrapper.
	 */
	public ConcreteTempNodeWrapper<? extends TempNode> getFirstChild() {
		if (!touched_firstchild) {
			firstchild = TempNodeWrapper.make(getReal().getFirstChild(), this);
			touched_firstchild = true;
		}
		TempNodeWrapper<? extends TempNode> x = firstchild;
		if (x != null && !(x instanceof ConcreteTempNodeWrapper<?>))
			x = x.getNextSibling();
		return (ConcreteTempNodeWrapper<? extends TempNode>)x;
	}
	
	/**
	 * Returns the first attribute node wrapper.
	 */
	public AttributeWrapper getFirstAttribute() {
		if (!touched_firstattribute) {
			firstattribute = AttrNodeWrapper.make(getReal().getFirstAttr(), this);
			touched_firstattribute = true;
		}
		AttrNodeWrapper<? extends AttrNode> a = firstattribute;
		if (a != null && !(a instanceof AttributeWrapper))
			a = a.getNextAttribute();
		return (AttributeWrapper)a;
	}

	@Override
	public void visitBy(WrapperNodeVisitor v) {
		v.visit(this);
	}
}
